package com.acme.randomstringwsproviderundertow.configuration;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xnio.Options;

import com.acme.randomstringwsproviderundertow.websocket.RandomStringWebSocketCallback;
import com.acme.randomstringwsproviderundertow.websocket.RandomStringWebSocketListener;
import io.undertow.Handlers;
import io.undertow.Undertow;
import io.undertow.UndertowOptions;
import io.undertow.websockets.core.WebSocketChannel;
import io.undertow.websockets.spi.WebSocketHttpExchange;

public class UndertowServerDeployment {

    private static final Logger LOGGER = LoggerFactory.getLogger( RandomStringWebSocketCallback.class );

    private final String host;
    private final int port;
    private final RandomStringWebSocketListener randomStringWebSocketListener;

    public UndertowServerDeployment( final String host, final int port ) {
        this.host = host;
        this.port = port;
        this.randomStringWebSocketListener = new RandomStringWebSocketListener();
    }

    public void start() {
        LOGGER.info( "Starting Undertow server on {} host and {} port", host, port );
        Undertow.builder()
                .addHttpListener( port, host )
                .setServerOption( UndertowOptions.NO_REQUEST_TIMEOUT, 100 * 1000 )
                .setServerOption( UndertowOptions.IDLE_TIMEOUT, 1000 * 1000 )
                .setSocketOption( Options.MAX_INBOUND_MESSAGE_SIZE, 4096 )
                .setSocketOption( Options.MAX_OUTBOUND_MESSAGE_SIZE, 4096 )
                .setHandler( Handlers.path().addExactPath( "/generate", Handlers.websocket( this::onConnect ) ) )
                .build()
                .start();
        LOGGER.info( "Undertow server started" );
    }

    private void onConnect( final WebSocketHttpExchange webSocketHttpExchange, final WebSocketChannel webSocketChannel ) {
        webSocketHttpExchange.setResponseHeaders( webSocketHttpExchange.getRequestHeaders() );
        webSocketChannel.getReceiveSetter().set( randomStringWebSocketListener );
        webSocketChannel.resumeReceives();
    }

}
