package com.acme.randomstringwsproviderundertow.websocket;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.RandomStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.undertow.websockets.core.AbstractReceiveListener;
import io.undertow.websockets.core.BufferedTextMessage;
import io.undertow.websockets.core.StreamSourceFrameChannel;
import io.undertow.websockets.core.WebSocketChannel;
import io.undertow.websockets.core.WebSockets;

public class RandomStringWebSocketListener extends AbstractReceiveListener {

    private static final Logger LOGGER = LoggerFactory.getLogger( RandomStringWebSocketListener.class );

    private final RandomStringWebSocketCallback randomStringWebSocketCallback;
    private final Map<Integer, String> cache;

    public RandomStringWebSocketListener() {
        randomStringWebSocketCallback = new RandomStringWebSocketCallback();
        cache = new ConcurrentHashMap<>();
    }

    @Override
    protected void onFullTextMessage( final WebSocketChannel channel, final BufferedTextMessage message ) throws IOException {
        super.onFullTextMessage( channel, message );
        final int length = Integer.parseInt( message.getData() ) * 1024;
        LOGGER.info( "Websocket message received to generate random string with length: {}", length );
        final String randomString = cache.computeIfAbsent( length, RandomStringUtils::randomAlphanumeric );
        WebSockets.sendText( randomString, channel, randomStringWebSocketCallback );
    }

    @Override
    protected void onClose( WebSocketChannel webSocketChannel, StreamSourceFrameChannel channel ) throws IOException {
        super.onClose( webSocketChannel, channel );
        LOGGER.info( "Websocket connection closed" );
    }

}
