package com.acme.wsapigatewaytomcat.configuration;

import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.config.annotation.EnableWebSocketMessageBroker;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketMessageBrokerConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketTransportRegistration;
import org.springframework.web.socket.server.HandshakeInterceptor;

import com.acme.wsapigatewaytomcat.websocket.ClientWebSocketConnectionHandler;
import com.github.mthizo247.cloud.netflix.zuul.web.socket.ZuulWebSocketProperties;

@Configuration
@EnableWebSocketMessageBroker
@EnableConfigurationProperties( {ZuulWebSocketProperties.class} )
public class ApiGatewayWebSocketZuulConfiguration implements WebSocketMessageBrokerConfigurer {

    private final ZuulWebSocketProperties zuulWebSocketProperties;
    private final HandshakeInterceptor webSocketHandshakeInterceptor;
    private final CorsProperties corsProperties;

    @Autowired
    public ApiGatewayWebSocketZuulConfiguration( final ZuulWebSocketProperties zuulWebSocketProperties, final HandshakeInterceptor webSocketHandshakeInterceptor, final CorsProperties corsProperties ) {
        this.zuulWebSocketProperties = zuulWebSocketProperties;
        this.webSocketHandshakeInterceptor = webSocketHandshakeInterceptor;
        this.corsProperties = corsProperties;
    }

    @Override
    public void configureWebSocketTransport( final WebSocketTransportRegistration registry ) {
        registry.addDecoratorFactory( ClientWebSocketConnectionHandler::new );
    }

    @Override
    public void registerStompEndpoints( final StompEndpointRegistry registry ) {
        for ( Map.Entry<String, ZuulWebSocketProperties.WsBrokerage> brokerageEntry : this.zuulWebSocketProperties.getBrokerages().entrySet() ) {
            final ZuulWebSocketProperties.WsBrokerage wsBrokerage = brokerageEntry.getValue();
            if ( wsBrokerage.isEnabled() ) {
                registry.addEndpoint( wsBrokerage.getEndPoints() )
                        .addInterceptors( webSocketHandshakeInterceptor )
                        .setAllowedOrigins( corsProperties.getAllowedOrigins().toArray( new String[corsProperties.getAllowedOrigins().size()] ) );
            }
        }
    }

}
