package com.acme.wsapigatewaytomcat.configuration;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.client.WebSocketClient;
import org.springframework.web.socket.server.HandshakeInterceptor;

import com.acme.wsapigatewaytomcat.websocket.WebsocketHandshakeInterceptor;
import com.github.mthizo247.cloud.netflix.zuul.web.proxytarget.ProxyTargetResolver;
import com.github.mthizo247.cloud.netflix.zuul.web.socket.ZuulWebSocketProperties;

@Configuration
public class WebsocketHandshakeInterceptorConfiguration {

    private final WebSocketClient webSocketClient;
    private final ProxyTargetResolver proxyTargetResolver;
    private final ZuulWebSocketProperties zuulWebSocketProperties;

    @Autowired
    public WebsocketHandshakeInterceptorConfiguration( final WebSocketClient webSocketClient, final ProxyTargetResolver proxyTargetResolver, final ZuulWebSocketProperties zuulWebSocketProperties ) {
        this.webSocketClient = webSocketClient;
        this.proxyTargetResolver = proxyTargetResolver;
        this.zuulWebSocketProperties = zuulWebSocketProperties;
    }

    @Bean
    public HandshakeInterceptor createWebSocketHandshakeInterceptor() {
        return new WebsocketHandshakeInterceptor( proxyTargetResolver, zuulWebSocketProperties, webSocketClient );
    }

}
