package com.acme.wsrandomstringload.simulation

import java.util.concurrent.TimeUnit

import com.acme.wsrandomstringload.configuration.RandomStringWebSocketRequestLoadConfiguration
import io.gatling.core.Predef._
import io.gatling.core.structure.ScenarioBuilder
import io.gatling.http.Predef._
import io.gatling.http.protocol.HttpProtocolBuilder

import scala.concurrent.duration.{DurationInt, FiniteDuration}

class RandomStringWebSocketRequestLoadSimulation extends Simulation {

  val randomStringWebSocketLoadConfiguration = new RandomStringWebSocketRequestLoadConfiguration()

  runScenario

  private def runScenario: SetUp = {
    val httpProtocol: HttpProtocolBuilder = http
      .baseURL("http://" + randomStringWebSocketLoadConfiguration.serverUrl + ":" + randomStringWebSocketLoadConfiguration.serverPort)
      .acceptHeader("text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8") // Here are the common headers
      .acceptEncodingHeader("gzip, deflate")
      .acceptLanguageHeader("en-US,en;q=0.5")
      .userAgentHeader("Gatling")
      .wsBaseURL("ws://" + randomStringWebSocketLoadConfiguration.serverUrl + ":" + randomStringWebSocketLoadConfiguration.serverPort)

    val randomStringWebSocketRequestLoadScenario: ScenarioBuilder =
      scenario("Random string websocket request load")
        .exec(ws("Connect websocket").open("/generate"))
        .exec(ws("Send random string websocket request")
          .sendText(randomStringWebSocketLoadConfiguration.randomStringLengthInKb)
          .check(wsAwait.within(new FiniteDuration(180, TimeUnit.SECONDS)).until(1).regex(".*")))
        .exec(ws("Close websocket").close)

    setUp(
      randomStringWebSocketRequestLoadScenario.inject(
        rampUsers(randomStringWebSocketLoadConfiguration.rampUpUsers) over (randomStringWebSocketLoadConfiguration.rampUpTime seconds)
      ).protocols(httpProtocol)
    )
  }

}
